// E3D_Utility.cpp: implementation of the E3D_Utility class.
// James Higgs 2002
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <math.h>
#include "e3d.h"
#include "E3D_Utility.h"

#define TRUE	1
#define FALSE	0

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

E3D_Utility::E3D_Utility()
{

}

E3D_Utility::~E3D_Utility()
{

}


/* vector_lengthf: 
 *  Floating point version of vector_length().
 */
float E3D_Utility::vector_length_f(float x, float y, float z)
{
   return sqrt(x*x + y*y + z*z);
}


/* normalize_vectorf: 
 *  Floating point version of normalize_vector().
 */
void E3D_Utility::normalize_vector_f(float *x, float *y, float *z)
{
   float length = 1.0f / vector_length_f(*x, *y, *z);

   *x *= length;
   *y *= length;
   *z *= length;
}

/* dot_product_f:
 *  Floating point version of dot_product().
 *  dot product of two vectors is a scalar value that can be used
 *  to calculate the angle between two vectors
 */
float E3D_Utility::dot_product_f(float x1, float y1, float z1, float x2, float y2, float z2)
{
    return x1*x2 + y1*y2 + z1*z2;
}

/* cos_theta_f:
 *  Get cos of the angle between 2 vectors
 */
float E3D_Utility::cos_theta_f(float x1, float y1, float z1, float x2, float y2, float z2)
{
	float dp;
	float vl1, vl2;
    dp = x1*x2 + y1*y2 + z1*z2;
	vl1 = vector_length_f(x1, y1, z1);
	vl2 = vector_length_f(x2, y2, z2);
	return dp / (vl1 * vl2);
}

/* cross_productf:
 *  Floating point version of cross_product().
 *  --> cross product of 2 vectors produces third vector that is orthogonal
 *  to both original vectors
 */
void E3D_Utility::cross_product_f(float x1, float y1, float z1, float x2, float y2, float z2, float *xout, float *yout, float *zout)
{
    *xout = (y1 * z2) - (z1 * y2);
    *yout = (z1 * x2) - (x1 * z2);
    *zout = (x1 * y2) - (y1 * x2);
}

/* polygon_z_normal_f:
 *  Floating point version of polygon_z_normal().
 */
float E3D_Utility::polygon_z_normal_f(Vector_f *v1, Vector_f *v2, Vector_f *v3)
{
   return ((v2->x-v1->x) * (v3->y-v2->y)) - ((v3->x-v2->x) * (v2->y-v1->y));
}


/* calculate normalised polygon normal
 * given 3 verts
 */
void E3D_Utility::poly_unit_normal_f(float x1, float y1, float z1, float x2, float y2, float z2, float *xout, float *yout, float *zout)
{
	float x, y, z;

	// calculate cross product to get normal vector
    x = (y1 * z2) - (z1 * y2);
    y = (z1 * x2) - (x1 * z2);
    z = (x1 * y2) - (y1 * x2);

	// make normal vector a unit vector
   float length = 1.0f / vector_length_f(x, y, z);

   *xout = x * length;
   *yout = y * length;
   *zout = z * length;

}
