/*
 * Fractal Landscape Demo
 * James Higgs 2003
 * Based on OpenGL base code from nehe.gamedev.net
 */

#include <windows.h>		// Header File For Windows
#include <stdio.h>			// Header File For Standard Input/Output
#include <math.h>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library
#include <gl\glfw.h>		// Header File for the GL Framwork Lib
#include "E3D.h"
#include "E3D_Scene1.h"
#include "E3D_Model.h"
#include "E3D_Object.h"
#include "E3D_Utility.h"
#include "fracland.h"

bool	active=TRUE;		// Window Active Flag Set To TRUE By Default
bool	fullscreen=TRUE;	// Fullscreen Flag Set To Fullscreen Mode By Default
bool	light;				// Lighting ON/OFF ( NEW )
bool	lp;					// L Pressed? ( NEW )

GLuint	texture[10];			// Storage For 10 Textures
GLuint	texid;
GLuint	model1;				// Storage For The Box Display List
GLuint	top;				// Storage For The Top Display List
GLuint	displist[20];
GLuint	xloop;				// Loop For X Axis
GLuint	yloop;				// Loop For Y Axis

GLfloat	xrot;				// Rotates Cube On The X Axis
GLfloat	yrot;				// Rotates Cube On The Y Axis
GLfloat zrot;
GLfloat zoom = 50;
GLfloat prop_rot;
GLfloat xpos[50], ypos[50], zpos[50];
GLfloat x_tree[200], y_tree[200], z_tree[200];
GLfloat z_gnd;
GLfloat y_eye = 50.0f;
GLfloat z_eye = -80.0f;
GLfloat x_eye = 0.0f;
GLfloat treescale = 1.0f;
GLfloat fogColour[] = { 0.2, 0.2, 0.2, 0.0 };
GLfloat LightAmbient[]= { 0.0f, 0.0f, 0.0f, 1.0f }; 			// Ambient Light Values
GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };				// Diffuse Light Values 
GLfloat LightPosition[]= { -0.3f, 0.7f, 0.3f, 0.0f };			// Light Position ( NEW )

static GLfloat boxcol[5][3]=
{
	{1.0f,0.0f,0.0f},{1.0f,0.5f,0.0f},{1.0f,1.0f,0.0f},{0.0f,1.0f,0.0f},{0.0f,1.0f,1.0f}
};

static GLfloat topcol[5][3]=
{
	{.5f,0.0f,0.0f},{0.5f,0.25f,0.0f},{0.5f,0.5f,0.0f},{0.0f,0.5f,0.0f},{0.0f,0.5f,0.5f}
};

int		mouse_x, mouse_y;
int		mouse_x_old, mouse_y_old;
int		mouse_dx, mouse_dy;

float currentx, currenty, currentz;
float targetx, targety, targetz;
float currentpitch, currentyaw, currentroll;
float targetpitch, targetyaw, targetroll;


class E3D_Scene Scene;
class E3D_Model gpModel[100];
class E3D_Object gpObject[100];

GLvoid BuildDispList(GLuint n, 	E3D_Model *pModel);


// build single display list (object)
GLvoid BuildDispList(GLuint n, 	E3D_Model *pModel)
{
	int i, p1, p2, p3, p4;

	glNewList(displist[n],GL_COMPILE);						// new list
		for(i=0; i < pModel->numpolys; i++) {
			// colour
			glColor3f(pModel->poly[i].colour.r, pModel->poly[i].colour.g, pModel->poly[i].colour.b);

			// handle triangles or quads
			switch(pModel->poly[i].numverts) {
				case 3 :
					p1 = pModel->poly[i].p[0];
					p2 = pModel->poly[i].p[2];
					p3 = pModel->poly[i].p[1];

					glBegin(GL_TRIANGLES);
						// normal
						glNormal3f(pModel->norm[i].x, pModel->norm[i].y, pModel->norm[i].z);
						// vertices
						glVertex3f(pModel->vert[p1].x, pModel->vert[p1].y, pModel->vert[p1].z);
						glVertex3f(pModel->vert[p2].x, pModel->vert[p2].y, pModel->vert[p2].z);
						glVertex3f(pModel->vert[p3].x, pModel->vert[p3].y, pModel->vert[p3].z);
					glEnd();
					break;
				case 4 :
					p1 = pModel->poly[i].p[0];
					p2 = pModel->poly[i].p[3];
					p3 = pModel->poly[i].p[2];
					p4 = pModel->poly[i].p[1];

					//glBindTexture( GL_TEXTURE_2D, texid );

					glBegin(GL_QUADS);
						// normal
						glNormal3f(pModel->norm[i].x, pModel->norm[i].y, pModel->norm[i].z);
						// vertices
						//glTexCoord2f( 0.0f,  0.0f );
						glVertex3f(pModel->vert[p1].x, pModel->vert[p1].y, pModel->vert[p1].z);
						//glTexCoord2f(  1.0f,  0.0f );
						glVertex3f(pModel->vert[p2].x, pModel->vert[p2].y, pModel->vert[p2].z);
						//glTexCoord2f(  1.0f, 1.0f );
						glVertex3f(pModel->vert[p3].x, pModel->vert[p3].y, pModel->vert[p3].z);
						//glTexCoord2f( 0.0f, 1.0f );
						glVertex3f(pModel->vert[p4].x, pModel->vert[p4].y, pModel->vert[p4].z);
					glEnd();
			} // end switch
		}
	glEndList();
}

// Build Cube Display Lists
GLvoid BuildLists()
{
	int i;
	printf("BuildLists()\n");

	displist[0]=glGenLists(20);							// Generate some display lists
	BuildDispList(0, Scene.pModel[0]);
	BuildDispList(1, Scene.pModel[1]);
/*
	for(i = 1; i < SECTION_MAX; i++) {
		displist[i]=displist[i-1]+1;							// Storage For "Top" Is "Box" Plus One
		//BuildDispList(i, Scene.pModel[i]);
	}
*/	
}

// build single display list (object)
void DrawModel(E3D_Model *pModel)
{
	int i, p1, p2, p3, p4;

	for(i=0; i < pModel->numpolys; i++) {
		// colour
		glColor3f(pModel->poly[i].colour.r, pModel->poly[i].colour.g, pModel->poly[i].colour.b);

		// handle triangles or quads
		switch(pModel->poly[i].numverts) {
			case 3 :
				p1 = pModel->poly[i].p[0];
				p2 = pModel->poly[i].p[2];
				p3 = pModel->poly[i].p[1];

				glBegin(GL_TRIANGLES);
					// normal
					glNormal3f(pModel->norm[i].x, pModel->norm[i].y, pModel->norm[i].z);
					// vertices
					//glTexCoord2f( 0.0f,  0.0f );
					glVertex3f(pModel->vert[p1].x, pModel->vert[p1].y, pModel->vert[p1].z);
					//glTexCoord2f(  1.0f,  0.0f );
					glVertex3f(pModel->vert[p2].x, pModel->vert[p2].y, pModel->vert[p2].z);
					//glTexCoord2f(  1.0f, 1.0f );
					glVertex3f(pModel->vert[p3].x, pModel->vert[p3].y, pModel->vert[p3].z);
				glEnd();
				break;
			case 4 :
				p1 = pModel->poly[i].p[0];
				p2 = pModel->poly[i].p[3];
				p3 = pModel->poly[i].p[2];
				p4 = pModel->poly[i].p[1];

				//glBindTexture( GL_TEXTURE_2D, texid );

				glBegin(GL_QUADS);
					// normal
					glNormal3f(pModel->norm[i].x, pModel->norm[i].y, pModel->norm[i].z);
					// vertices
					//glTexCoord2f( 0.0f,  0.0f );
					glVertex3f(pModel->vert[p1].x, pModel->vert[p1].y, pModel->vert[p1].z);
					//glTexCoord2f(  1.0f,  0.0f );
					glVertex3f(pModel->vert[p2].x, pModel->vert[p2].y, pModel->vert[p2].z);
					//glTexCoord2f(  1.0f, 1.0f );
					glVertex3f(pModel->vert[p3].x, pModel->vert[p3].y, pModel->vert[p3].z);
					//glTexCoord2f( 0.0f, 1.0f );
					glVertex3f(pModel->vert[p4].x, pModel->vert[p4].y, pModel->vert[p4].z);
				glEnd();
		} // end switch
	}
}

// build single display list (object)
void DrawStar(void)
{

	// colour
	glColor3f(1.0f, 1.0f, 1.0f);

	//glBindTexture( GL_TEXTURE_2D, texid );

	glBegin(GL_QUADS);
		// normal

		glTexCoord2f( 0.0f,  0.0f );
		glVertex3f(-1.0, 1.0, 0.0);
		glTexCoord2f(  1.0f,  0.0f );
		glVertex3f(-1.0, -1.0, 0.0);
		glTexCoord2f(  1.0f, 1.0f );
		glVertex3f(1.0, -1.0, 0.0);
		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f(1.0, 1.0, 0.0);
	glEnd();
}


int LoadGLTextures()									// Load Bitmaps And Convert To Textures
{
    // Get and select a texture object ID
    glGenTextures( 10, &texture[0] );
    glBindTexture( GL_TEXTURE_2D, texture[0] );

    // Load texture from file, and build all mipmap levels. The
    // texture is automatically uploaded to texture memory.
    if( !glfwLoadTexture2D( "tahoma.tga", GLFW_BUILD_MIPMAPS_BIT ) )
    {
		MessageBox(NULL, "Could not load texture!!!", "!!!",MB_YESNO|MB_ICONQUESTION);
        glfwTerminate();
        return 0;
    }

    // Use trilinear interpolation for shrinking tex (GL_LINEAR_MIPMAP_LINEAR)
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR_MIPMAP_LINEAR );
    // Use bilinear interpolation for magnifying tex (GL_LINEAR)
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                     GL_LINEAR );

    glBindTexture( GL_TEXTURE_2D, texture[1] );
    // Load texture from file, and build all mipmap levels. The
    // texture is automatically uploaded to texture memory.
    if( !glfwLoadTexture2D( "star.tga", GLFW_BUILD_MIPMAPS_BIT ) )
    {
		MessageBox(NULL, "Could not load texture!!!", "!!!",MB_YESNO|MB_ICONQUESTION);
        glfwTerminate();
        return 0;
    }

	// Typical Texture Generation Using Data From The Bitmap
	//	glBindTexture(GL_TEXTURE_2D, texture[0]);
	//	glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	//	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	return 1;										// Return The Status
}

GLvoid ReSizeGLScene(GLsizei width, GLsizei height)		// Resize And Initialize The GL Window
{
	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	// JH - changed far z here to 500
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,1.0f,500.0f);

	// set camera position and direction
	//gluLookAt(0, 20, 0, 0, 0, 50, 0, 1, 0); 

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}

void GLFWCALL ResizeWindow( int width, int height ) {
	//glfwSetWindowSize( width, height );

	if (height==0)										// Prevent A Divide By Zero By
	{
		height=1;										// Making Height Equal One
	}

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	// JH - changed far z here to 500
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,1.0f,500.0f);

	// set camera position and direction
	//gluLookAt(0, 20, 0, 0, 0, 50, 0, 1, 0); 

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

}


int InitGL(GLvoid)										// All Setup For OpenGL Goes Here
{
	printf("InitGL()\n");

    // Enable texturing
    glEnable( GL_TEXTURE_2D );
	
	if (!LoadGLTextures())								// Jump To Texture Loading Routine
	{
		return FALSE;									// If Texture Didn't Load Return FALSE
	}

	glShadeModel(GL_SMOOTH);							// FLAT / SMOOTH shading
	glClearColor(0.2f, 0.2f, 0.2f, 0.5f);				// Background colour
	glClearDepth(1.0f);									// Depth Buffer Setup

	glFogi(GL_FOG_MODE, GL_LINEAR);						// Fog Mode
	glFogfv(GL_FOG_COLOR, fogColour);					// Set Fog Color
	glFogf(GL_FOG_DENSITY, 0.35f);						// How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);					// Fog Hint Value
	glFogf(GL_FOG_START, 1.0f);							// Fog Start Depth
	glFogf(GL_FOG_END, 400.0f);							// Fog End Depth
	//glEnable(GL_FOG);									// Enables GL_FOG

	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LESS);								// The Type Of Depth Testing To Do
	//glDepthRange(0.5, 200.0);
	glEnable(GL_COLOR_MATERIAL);						// Enable Material Coloring
	//glEnable(GL_POLYGON_SMOOTH);

	//glPolygonMode(GL_FRONT, GL_LINE);
	//glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	//glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	// lighting
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);		// set ambient light level
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		// set light colour
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	// set light position or direction
	glEnable(GL_LIGHT1);								// Enable Light One
	glEnable(GL_LIGHTING);								// Enable Lighting
	glEnable(GL_NORMALIZE);								// fix normals after scaling

	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Set The Blending Function For Translucency

	return TRUE;										// Initialization Went OK
}

// texture font print
void jprint(char *text, float x, float y) {
	int c;
	float tx, ty;
	char *pc;
	pc = text;

	while( *pc != 0 ) {
		c = *pc - 32;
		if(c < 0) c = 0;
		tx = float( c % 16 ) * 0.0625;
		ty = float( c / 16 ) * 0.0625;
		glBegin(GL_QUADS);
			// normal
			//glNormal3f(0.0f, 0.0f, -1.0f);
			// vertices
			glTexCoord2f( tx,  1.0 - ty );
			glVertex3f(x, y, 0);
			glTexCoord2f(  tx,  1.0 - ty - 0.0625 );
			glVertex3f(x, y+16, 0);
			glTexCoord2f(  tx + 0.0625, 1.0 - ty - 0.0625);
			glVertex3f(x+16, y+16, 0);
			glTexCoord2f( tx + 0.0625, 1.0 - ty );
			glVertex3f(x+16, y, 0);
		glEnd();
		x += 12;
		pc++;
	}

}

// render our scene
int DrawGLScene(GLvoid)									// Here's Where We Do All The Drawing
{
	int i, j, i1, j1, numfracpiecesdrawn;
	float scale = 10.0f;
	float x, z;
	float nx, ny, nz;
	int steps = 50;
	float h_scale = 2.0;				// horiz size of each fractal piece
	float v_scale = 0.4;				// vertical fractal scale
	float dist, dx, dz;
	char stemp[512];
	E3D_Object *pO;
	E3D_Utility *pUtil;
	int retval;
	GLdouble modelMatrix[16];
	GLdouble projMatrix[16];
	GLint viewport[4];
	GLdouble winx, winy, winz;
	float c, s;

    // Enable scissor test
    glDisable( GL_SCISSOR_TEST );
	glDisable(GL_BLEND);

	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing


	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

    // Enable scissor test
    //glEnable( GL_SCISSOR_TEST );

    // Main View
    //glViewport( 0, 120, 800, 360 );
    //glScissor( 0, 120, 800, 360 );
    glViewport( 160, 0, 640, 480 );
    glScissor( 160, 0, 640, 480 );


	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix


	// Calculate The Aspect Ratio Of The Window
	// JH - changed far z here to 500
	gluPerspective(45.0f,(GLfloat)640/480,1.0f,1000.0f);

	//x_eye = currentx + sin(yrot / RAD2DEG) * zoom;
	//z_eye = currentz + cos(yrot / RAD2DEG) * zoom;
	gluLookAt(x_eye, y_eye, z_eye, x_eye + sin(yrot), y_eye, z_eye + cos(yrot), 0, 1, 0); 


	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	// switch off texturing
	glBindTexture(GL_TEXTURE_2D, 0);

	// draw grid
	glColor3f(0.5f, 0.5f, 0.5f);
    glBegin( GL_LINES );
	    // Horizontal lines
		x = scale * 0.5f * (float)(steps-1);
		z = -scale * 0.5f * (float)(steps-1);
		for( i = 0; i < steps; i ++ )
		{
			glVertex3f( -x, 0.0f, z );
			glVertex3f( x, 0.0f, z );
			z += scale;
		}

		// Vertical lines
		x = -scale * 0.5f * (float)(steps-1);
		z = scale * 0.5f * (float)(steps-1);
		for( i = 0; i < steps; i ++ )
		{
			glVertex3f( x, 0.0f, -z );
			glVertex3f( x, 0.0f, z );
			x += scale;
		}
    glEnd();

	/*
	// draw landscape
	glColor3f(0.8f, 0.8f, 0.5f);
    glBegin( GL_LINES );
		for(i=0; i<120; i++) {
			for(j=0; j<120; j++) {
				glVertex3f((GLfloat)i, Map[i][j] / 5, (GLfloat)(j));
				glVertex3f((GLfloat)i, Map[i][j+1] / 5 , (GLfloat)(j+1));
			}
		}
    glEnd();
*/
	glDisable(GL_LIGHTING);

	pUtil = new E3D_Utility;


	// precompute current eye direction sin and cos values
	c = cos(yrot);
	s = sin(yrot);

	//glColor3f(0.8f, 0.5f, 0.3f);
	glColor3b(100, 64, 32);
    glBegin( GL_TRIANGLES );
	// process fractal landscape in 16x16 blocks, 
	// checking individual blocks for visibility,
	// and changing LOD according to distance
	numfracpiecesdrawn = 0;
	for(i1=0; i1 < 504; i1 += 8) {
		for(j1=0; j1 < 504; j1 += 8) {
			// get distance to this block from the eye
			dx = i1*h_scale - x_eye;
			dz = j1*h_scale - z_eye;
			dist = dx*dx + dz*dz;	// optimise

			if(dist < 40000) {
				// check if this object will appear on screen
				// (ie: rotate it about the eye)
				winx = c*dx + s*dz;
				winz = s*dx + c*dz;
				if((winz > (0.9*winx - 20.0f)) && (winz > (-0.9*winx - 20.0f))) {
					for(i=i1; i<(i1+8); i++) {
						for(j=j1; j<(j1+8); j++) {


		/*						// normal
							pUtil->poly_unit_normal_f(h_scale, ((float)Map[i+1][j+1] / 5) - ((float)Map[i][j] / 5), h_scale,
								h_scale, ((float)Map[i][j+1] / 5) - ((float)Map[i][j] / 5), 0,
								&nx, &ny, &nz);
							glNormal3f(nx, ny, nz);
		*/						// tri
							glColor3b(MapColour[i][j][0],
									MapColour[i][j][1],
									MapColour[i][j][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j] * v_scale, (GLfloat)(j)*h_scale);
							glColor3b(MapColour[i][j+1][0],
									MapColour[i][j+1][1],
									MapColour[i][j+1][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j+1] * v_scale , (GLfloat)(j+1)*h_scale);
							glColor3b(MapColour[i+1][j+1][0],
									MapColour[i+1][j+1][1],
									MapColour[i+1][j+1][2]);
							glVertex3f((GLfloat)(i+1)*h_scale, (float)Map[i+1][j+1] * v_scale , (GLfloat)(j+1)*h_scale);

		/*						// normal
							pUtil->poly_unit_normal_f(1, (Map[i+1][j+1] / 5) - (Map[i][j] / 5), 1,
								1, (Map[i][j+1] / 5) - (Map[i][j] / 5), 0,
								&nx, &ny, &nz);
							glNormal3f(nx, ny, nz);
		*/
							// tri
							glColor3b(MapColour[i][j][0],
									MapColour[i][j][1],
									MapColour[i][j][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j] * v_scale, (GLfloat)(j)*h_scale);
							glColor3b(MapColour[i+1][j+1][0],
									MapColour[i+1][j+1][1],
									MapColour[i+1][j+1][2]);
							glVertex3f((GLfloat)(i+1)*h_scale, (float)Map[i+1][j+1] * v_scale , (GLfloat)(j+1)*h_scale);
							glColor3b(MapColour[i+1][j][0],
									MapColour[i+1][j][1],
									MapColour[i+1][j][2]);
							glVertex3f((GLfloat)(i+1)*h_scale, (float)Map[i+1][j] * v_scale , (GLfloat)(j)*h_scale);
							numfracpiecesdrawn++;
						}
					}
				} // end if winx > winz
			} // end if dist < n
			else if(dist < 80000) {
				// check if this object will appear on screen
				// (ie: rotate it about the eye)
				winx = c*dx + s*dz;
				winz = s*dx + c*dz;
				if((winz > (0.8*winx)) && (winz > (-0.8*winx))) {
					for(i=i1; i<(i1+8); i += 2) {
						for(j=j1; j<(j1+8); j += 2) {

							// normal
							//pUtil->poly_unit_normal_f(h_scale*2, ((float)Map[i+2][j+2] / 5) - ((float)Map[i][j] / 5), h_scale*2,
							//	h_scale*2, ((float)Map[i][j+2] / 5) - ((float)Map[i][j] / 5), 0,
							//	&nx, &ny, &nz);
							//glNormal3f(nx, ny, nz);
							// tri
							glColor3b(MapColour[i][j][0],
									MapColour[i][j][1],
									MapColour[i][j][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j] * v_scale, (GLfloat)(j)*h_scale);
							glColor3b(MapColour[i][j+1][0],
									MapColour[i][j+1][1],
									MapColour[i][j+1][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j+2]  * v_scale , (GLfloat)(j+2)*h_scale);
							glColor3b(MapColour[i+1][j+1][0],
									MapColour[i+1][j+1][1],
									MapColour[i+1][j+1][2]);
							glVertex3f((GLfloat)(i+2)*h_scale, (float)Map[i+2][j+2]  * v_scale , (GLfloat)(j+2)*h_scale);

							// normal
		//						pUtil->poly_unit_normal_f(1, (Map[i+1][j+1] / 5) - (Map[i][j] / 5), 1,
		//							1, (Map[i][j+1] / 5) - (Map[i][j] / 5), 0,
		//							&nx, &ny, &nz);
							glNormal3f(nx, ny, nz);
							// tri
							glColor3b(MapColour[i][j][0],
									MapColour[i][j][1],
									MapColour[i][j][2]);
							glVertex3f((GLfloat)i*h_scale, (float)Map[i][j] * v_scale, (GLfloat)(j)*h_scale);
							glColor3b(MapColour[i+1][j+1][0],
									MapColour[i+1][j+1][1],
									MapColour[i+1][j+1][2]);
							glVertex3f((GLfloat)(i+2)*h_scale, (float)Map[i+2][j+2]  * v_scale , (GLfloat)(j+2)*h_scale);
							glColor3b(MapColour[i+1][j][0],
									MapColour[i+1][j][1],
									MapColour[i+1][j][2]);
							glVertex3f((GLfloat)(i+2)*h_scale, (float)Map[i+2][j]  * v_scale , (GLfloat)(j)*h_scale);
							numfracpiecesdrawn++;
						}
					}
				} // end if winx > winz
			} // end else if long dist
		} // next j1
	} // next i1
    glEnd();
 
	// do this to get lighting working correctly on non-textured polys
	//glBindTexture(GL_TEXTURE_2D, texid);
	//glPolygonMode(GL_FRONT, GL_LINE);

	// NB: it seems like you have to specify your xformations in reverse order (!!!)
	// !!!wierd!!!
	// draw objects in scene
	glEnable(GL_LIGHTING);
	for(i=0; i<Scene.numObjects; i++) {
		pO = Scene.pObject[i];

		dx = pO->x - x_eye;
		dz = pO->z - z_eye;
		dist = dx*dx + dz*dz;	// optimise

		// filter instance to view
		if(dist < 80000) {
			// check if this object will appear on screen
			// (ie: rotate it about the eye)
			winx = c*dx + s*dz;
			winz = s*dx + c*dz;
			if((winz > (0.9*winx - 20.0f)) && (winz > (-0.9*winx - 20.0f))) {
				glLoadIdentity();
				glTranslatef(pO->x, pO->y, pO->z);
				//glRotatef(pO->ry, 0.0f, 1.0f, 0.0f);			// Spin Left And Right
				glScalef(pO->sx, pO->sy, pO->sz);
				//glCallList(displist[pO->modelNumber]);			// Draw the object
				DrawModel(pO->pModel);
			}
		}
	}

	// draw decal stars
	// NB: have to be sorted back to front first!
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBindTexture( GL_TEXTURE_2D, texture[1] );
	for(i=0; i<Scene.numObjects; i++) {
		pO = Scene.pObject[i];

		dx = pO->x - x_eye;
		dz = pO->z - z_eye;
		dist = dx*dx + dz*dz;	// optimise

		// filter instance to view
		if(dist < 80000) {
			// check if this object will appear on screen
			// (ie: rotate it about the eye)
			winx = c*dx + s*dz;
			winz = s*dx + c*dz;
			if((winz > (0.9*winx - 20.0f)) && (winz > (-0.9*winx - 20.0f))) {
				glLoadIdentity();
				glTranslatef(pO->x, pO->y, pO->z);
				//glCallList(displist[pO->modelNumber]);			// Draw the object
				//DrawModel(pO->pModel);
				glRotatef((yrot + 3.1415) * 54, 0.0f, 1.0f, 0.0f);			// Spin Left And Right
				glScalef(10.0, 10.0, 10.0);
				DrawStar();
			}
		}
	}

	glDisable(GL_BLEND);

    // Sidebar
    glViewport( 0, 0, 160, 480 );
    glScissor( 0, 0, 160, 480 );
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	gluOrtho2D(0, 160, 480, 0);							// Orthographic projection

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	//glTranslatef(0.0f,0.0f,5.0f);						// Move One Unit Into The Screen
	glColor3f(1.0f,1.0f,1.0f);

	glPolygonMode(GL_FRONT_AND_BACK , GL_FILL);
	glBindTexture( GL_TEXTURE_2D, texture[0] );

	glBegin(GL_QUADS);
		// vertices
		glTexCoord2f( 0.0f,  0.25f );
		glVertex3f(0, 0, 0);
		glTexCoord2f( 1.0f,  0.25f );
		glVertex3f(800, 0, 0);
		glTexCoord2f( 1.0f,  0.5f );
		glVertex3f(800, 120, 0);
		glTexCoord2f( 0.0f,  0.5f );
		glVertex3f(0, 120, 0);
	glEnd();

	glBegin(GL_QUADS);
		// vertices
		glTexCoord2f( 0.0f,  0.50f );
		glVertex3f(0, 0, 0);
		glTexCoord2f( 1.0f,  0.50f );
		glVertex3f(800, 0, 0);
		glTexCoord2f( 1.0f,  1.0f );
		glVertex3f(800, 120, 0);
		glTexCoord2f( 0.0f,  1.0f );
		glVertex3f(0, 120, 0);
	glEnd();

	//glColor3f(1.0f,1.0f,1.0f);
	//glBindTexture( GL_TEXTURE_2D, texture[0] );
	//glBindTexture( GL_TEXTURE_2D, 0);
	jprint("Tracker V1.0", 0, 0);

	jprint("Current:", 20, 20); //was 0, 20
	sprintf(stemp, "X     %3.3f", x_eye);
	jprint(stemp, 0, 36);
	sprintf(stemp, "Y     %3.3f", y_eye);
	jprint(stemp, 0, 52);
	sprintf(stemp, "Z     %3.3f", z_eye);
	jprint(stemp, 0, 68);
	sprintf(stemp, "Pitch %3.3f", currentpitch);
	jprint(stemp, 0, 84);
	sprintf(stemp, "Yaw   %3.3f", currentyaw);
	jprint(stemp, 0, 100);
	sprintf(stemp, "Roll  %3.3f", currentroll);
	jprint(stemp, 0, 116);

	jprint("Next:", 0, 144);
	sprintf(stemp, "Pitch %3.3f", winx);
	jprint(stemp, 0, 160);
	sprintf(stemp, "Yaw   %3.3f", winy);
	jprint(stemp, 0, 176);
	sprintf(stemp, "Roll  %3.3f", winz);
	jprint(stemp, 0, 192);

	sprintf(stemp, "Mouse X %d", mouse_x);
	jprint(stemp, 0, 400);
	sprintf(stemp, "Mouse Y %d", mouse_y);
	jprint(stemp, 0, 416);

	sprintf(stemp, "Polys: %d", numfracpiecesdrawn);
	jprint(stemp, 0, 460);

	return TRUE;						// Jump Back
}

// get mouse increments
void GetMouseChange() {
	mouse_x_old = mouse_x;
	mouse_y_old = mouse_y;
	glfwGetMousePos(&mouse_x, &mouse_y);
	mouse_dx = mouse_x - mouse_x_old;
	mouse_dy = mouse_y - mouse_y_old;
	if(mouse_dx > 10) mouse_dx = 10;
	if(mouse_dy > 10) mouse_dy = 10;
}


// keyboard input handler
void GLFWCALL KeyboardHandler( int key, int action ) {

	if(action == GLFW_PRESS) {
		switch(key) {
		case GLFW_KEY_F9 :		// save track
			//SaveTrack();
			break;
		case GLFW_KEY_ENTER :	// add section
			//NewSection();
			//UpdateSection(numsections-1);
			break;
		case GLFW_KEY_DEL :		// add section
			//DeleteSection();
			break;
		}
	}
}

int main(void) {

	BOOL	done=FALSE;								// Bool Variable To Exit Loop
	short i;
	char stemp[256];
	int debounce = 0;

    int     width, height, running, frames, x, y;
    double  t, t0, fps;
    char    titlestr[ 200 ];
	int		scr_mode;
	E3D_Object *pObject;

	scr_mode = GLFW_WINDOW;
/*
	// Ask The User Which Screen Mode They Prefer
	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDYES)
	{
		scr_mode = GLFW_FULLSCREEN;
	}
*/
	Scene.Load("out.scn");
	//Scene.Start();
	//Scene.AddModel("ufo.3do");



/*
	sprintf(stemp, "%d models and %d objects in scene", Scene.numModels, Scene.numObjects);
	MessageBox(NULL, stemp, "!!!",MB_YESNO|MB_ICONQUESTION);
*/
    // Initialise GLFW
    if(glfwInit() != GL_TRUE) return(1);

    // Open OpenGL window
    if( !glfwOpenWindow( 800, 480, 0,0,0,0, 0,0, scr_mode ) )
    {
        glfwTerminate();
        return 0;
    }
    glfwSetWindowTitle( "Return to Fractalus" );

	glfwSetWindowSizeCallback( ResizeWindow );

	//printf("Refresh rate: %d \n", glfwGetWindowParam(GLFW_REFRESH_RATE));
	//printf("ext1: %d \n", glfwExtensionSupported("GL_ARB_multitexture"));

    // Enable sticky keys
    //glfwEnable( GLFW_STICKY_KEYS );

    // Disable vertical sync (on cards that support it)
    glfwSwapInterval( 1 );

	// Set up our OpenGL env and build our display lists
	printf( "InitGL() returned %d\n", InitGL() );
	BuildLists();

    // Swap buffers
    glfwSwapBuffers();
	// get mouse DX/Y
	GetMouseChange();

	//InitSections();
	//NewSection();
	//UpdateSection(0);

	// set keyboard callback
	glfwSetKeyCallback( KeyboardHandler );


	InitFract();
	CreateFract();
	ShadeFract();

	for(i=0; i<100; i++) {
		pObject = Scene.AddObject("tree.3do");
		x = rand()%200;
		y = rand()%200;
		pObject->x = (float)(x) * 2.0f;
		pObject->z = (float)(y) * 2.0f;
		pObject->y = (float)(Map[x][y]) * 0.4;
	}

    // Main loop
    running = GL_TRUE;
    frames = 0;
    t0 = glfwGetTime();
    while( running ) {

        // Get time and mouse position
        t = glfwGetTime();
        glfwGetMousePos( &x, &y );

        // Calculate and display FPS (frames per second)
        if( (t-t0) > 1.0 || frames == 0 )
        {
            fps = (double)frames / (t-t0);
            sprintf( titlestr, "Fracatalus (GLFW Version) (%.1f FPS)", fps );
            glfwSetWindowTitle( titlestr );
            t0 = t;
            frames = 0;
        }
        frames ++;

		DrawGLScene();

        // Swap buffers
        glfwSwapBuffers();

        // Check if the ESC key was pressed or the window was closed
        running = !glfwGetKey( GLFW_KEY_ESC ) &&
                  glfwGetWindowParam( GLFW_OPENED );

		if (glfwGetKey(GLFW_KEY_LEFT))				// Left Arrow Being Pressed?
		{
			yrot += 0.02f;
			//x_eye -= 0.5f;
		}
		if (glfwGetKey(GLFW_KEY_RIGHT))				// Right Arrow Being Pressed?
		{
			yrot -= 0.02f;
			//x_eye += 0.5f;
		}
		if (glfwGetKey(GLFW_KEY_UP))				// Up Arrow Being Pressed?
		{
			x_eye += sin(yrot);				// Move in
			z_eye += cos(yrot);
		}
		if (glfwGetKey(GLFW_KEY_DOWN))				// Down Arrow Being Pressed?
		{
			x_eye -= sin(yrot);				// Move in
			z_eye -= cos(yrot);
		}

		if (glfwGetKey('A'))				// Up Arrow Being Pressed?
		{
			y_eye += 0.5f;				// Move in
		}
		if (glfwGetKey('Z'))				// Down Arrow Being Pressed?
		{
			y_eye -= 0.5f;				// Move out
		}



		// get mouse DX/Y
		GetMouseChange();
		// get Mouse Button
		if(glfwGetMouseButton(GLFW_MOUSE_BUTTON_LEFT) == GLFW_PRESS) {
			if(mouse_x < 160) {
				i = mouse_y / 16;
				switch(i) {
				case 10 : // pitch up
					//if(ts[numsections-1].endpitch < 45.0f) ts[numsections-1].endpitch += 0.2f;
					break;
				case 11 : // yaw up
					//ts[numsections-1].endyaw += 0.2f;
					break;
				case 12 : // roll up
					//if(ts[numsections-1].endroll < 45.0f) ts[numsections-1].endroll += 0.2f;
					break;
				}
				//UpdateSection(numsections -1);
			}
		}
		if(glfwGetMouseButton(GLFW_MOUSE_BUTTON_RIGHT) == GLFW_PRESS) {
			if(mouse_x < 160) {
				i = mouse_y / 16;
				switch(i) {
				case 10 : // pitch up
					//if(ts[numsections-1].endpitch > -45.0f) ts[numsections-1].endpitch -= 0.2f;
					break;
				case 11 : // yaw up
					//ts[numsections-1].endyaw -= 0.2f;
					break;
				case 12 : // roll up
					//if(ts[numsections-1].endroll > -45.0f) ts[numsections-1].endroll -= 0.2f;
					break;
				}
				//UpdateSection(numsections -1);
			}
		}

		if(debounce) debounce--;
	} // wend

    // Close OpenGL window and terminate GLFW
    glfwTerminate();
	return 0;									// Exit The Program
}

